/***************************************************************************
 *   Copyright (C) 2006 by Jens Koehler                                    *
 *   irimi_at_gmx_de                                                       *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <unistd.h>
#include <getopt.h>
#include <iostream>
#include <cstdlib>
#include <string>
#include <fstream>

#include "amarokserver.h"
#include "commands.h"
#include "debugTracer.h"

using namespace std;


void check_optargs(const char *opt);
void show_header();
void show_help();
bool readConfigfile (const char* file,string &user, string &pw,int &port,bool &daemon, bool &nocheck, bool& info);
void start_server(const char* user, const char *pw, int port, bool nocheck);

int main(int argc, char *argv[])
{
    const char *user=0;
    const char *pw=0;
    int port = VDRAMGW_PORT;
    bool daemon = false;
    bool nocheck = false;
    bool showinfo=false;

    string strUser="User";
    string strPw="Pw";

    const char *userfile=0;
    const char* deffile="/etc/vdr/plugin/amarok/vdramgw.conf";

    if (argc==1) // use default config file
    {
        if (!readConfigfile (deffile,strUser, strPw,port,daemon,nocheck,showinfo))
        {
            cerr << "Error, reading config file \"" << deffile << "\"" << endl;
            return -1;
        }

        user=strUser.c_str();
        pw=strPw.c_str();
    }
    else // command line parser
    {
        const char * short_options="hndP:u:p:c:i";
        struct option long_options[] =
            {
                {"help", 0, 0, 'h'
                },
                {"nocheck", 0, 0, 'n' },
                {"daemon", 0, 0, 'd' },
                {"Port", 1, 0, 'P' },
                {"user", 1, 0, 'u' },
                {"passwd", 1, 0, 'p' },
                {"config", 1, 0, 'c' },
                {"info", 0, 0, 'i' },
                {0, 0, 0, 0}
            };

        int next_opt=0;
        int *option_index=NULL;
        opterr = 0; // reset
        while ((next_opt = ::getopt_long (argc, argv, short_options,long_options, option_index)) != -1)
        {
            //cout << " ** optarg=" << optarg << endl;
            switch (next_opt)
            {
            case ':':
            case '?':
                cerr << "Error, unknown argument \"-" << optopt << "\"." << endl;
                show_help();
                return 1;
            case 'h' :
                show_help();
                return 1;
            case 'd' :
                daemon=true;
                break;
            case 'i' :
                showinfo=true;
                break;
            case 'n' :
                nocheck=true;
                break;
            case 'u' :
                check_optargs(optarg);
                user = optarg;
                break;
            case 'p' :
                check_optargs(optarg);
                pw = optarg;
                break;
            case 'c' :
                check_optargs(optarg);
                userfile = optarg;
                break;
            case 'P' :
                char* endptr = 0L;
                check_optargs(optarg);
                port = strtol (optarg, &endptr, 10);

                if ((endptr == 0L) || (*endptr != 0L))
                {
                    cerr << "wrong port:" << optarg << endl;
                    show_help ();
                    return 1;
                }
                break;

            }
        }
    } // command line parser ends

    if (userfile)
    {
        if (!readConfigfile (userfile,strUser, strPw,port,daemon,nocheck, showinfo))
        {
            cerr << "Error, reading config file \"" << userfile << "\"" << endl;

            return -1;
        }

        user=strUser.c_str();
        pw=strPw.c_str();
    }

    if (!user || !pw)
    {
        cerr << "Server login name and/or passward isn't set" << endl;
        show_help();
        return 1;
    }

    if (showinfo)
        AMAROK_TRACER_LEVEL_THRESHOLD=AMAROK_TRACE_INFO;

    if (daemon)
    {
        int pid = fork();
        if (pid == -1)
        {
            cerr << "cannot start in daemon mode" << endl;
            return -1;
        }
        if (pid == 0)
        { // child
            start_server(user,pw,port,nocheck);
            return 0;

        }
        else
        { // parent
            cout << "VDRAmarokGateway " << VDRAMGW_VERSION
            << " has started with PID=" << pid << endl;
            return 0; //waitpid(pid, &status, 0);
        }
    }
    else
    {

        start_server(user,pw,port,nocheck);
        return 0;

    }
}

void check_optargs(const char *opt)
{

    if (strlen (opt) <= 0)
    {
        cerr << "missing argument" << endl;
        show_help ();
        exit (1);
    }
}

void start_server(const char* user, const char *pw, int port, bool nocheck)
{
    AmarokServer *server = 0;
    server = new AmarokServer (user,pw);
    bool ok=false;

    if (!nocheck)
        ok=server->checkVersion();
    else
    {
        cout << "** amaroK version check disabled ! **\nYou MUST use amaroK 1.4.x !\n";
        ok=true;
    }

    if (ok)
    {
        cout << "Starting server listening port " << port << endl;
        server->runServer(port);
        ::sleep(10); // wait 10s due to WAIT_TIME client has disconnected
        cout << "server has stopped !" << endl;

    }
    else
        cerr << "start of server has failed !" << endl;


    delete server;
    server=0;
}

bool readConfigfile (const char* file, string &user,  string &pw,int &port,bool &daemon, bool &nocheck, bool &info)
{

    string      strLine;

    bool bRet=false;

    ifstream in(file);
    if (!in)
    {
        cerr << "Cannot open config file:" << file << endl;
    }
    else
    {
        unsigned int index=0;
        while(in && getline(in,strLine) && !in.eof())
        {
            switch (strLine[0])
            {
            case 'p':
                if (strLine.find("password=")==0 && strLine.length()>=10)
                {
                    index=strLine.find_first_of("=")+1;
                    pw=strLine.substr(index,strLine.length()-(index));
                    bRet = true;
                }
                else if (strLine.find("port=")==0 && strLine.length()>=6)
                {
                    index=strLine.find_first_of("=")+1;
                    char* endptr = 0L;
                    port=(int)strtol(strLine.substr(index,strLine.length()-(index)).c_str(),&endptr,10);
                    bRet = true;
                    //if ((endptr == 0L) || (*endptr != 0L))
                    //   port=VDRAMGW_PORT;
                }
                break;
            case 'u':
                if (strLine.find("user=")==0 && strLine.length()>=6)
                {
                    index=strLine.find_first_of("=")+1;
                    user=strLine.substr(index,strLine.length()-(index));
                    bRet = true;
                }
                break;
            case 'n':
                if (strLine.find("nocheck=")==0 && strLine.length()>=6)
                {
                    bRet = true;
                    index=strLine.find_first_of("=")+1;
                    if(strLine.substr(index,strLine.length()-(index)).find("true")==0)
                        nocheck=true;
                }
                break;
            case 'd':
                if (strLine.find("daemon=")==0 && strLine.length()>=7)
                {
                    bRet = true;
                    index=strLine.find_first_of("=")+1;
                    if(strLine.substr(index,strLine.length()-(index)).find("enabled")==0)
                        daemon=true;

                }
                break;
            case 'i':
                if (strLine.find("info=")==0 && strLine.length()>=7)
                {
                    bRet = true;
                    index=strLine.find_first_of("=")+1;
                    if(strLine.substr(index,strLine.length()-(index)).find("enabled")==0)
                        info=true;

                }
                break;
            case '\n':
            case '#':
            default:
                break;
            }
        }
    }
    return bRet;

}


void show_header()
{
    cout << "VDRAmarokGateway for vdr-amarok plugin by irimi_at_gmx_de" << endl;
    cout << "Version " <<  VDRAMGW_VERSION <<" / "<< __DATE__ << " / " << __TIME__ << endl;
}


void show_help ()
{
    show_header();

    cerr <<  "\nusage: vdramgw [-h] [-d] [-u 'user'] [-p 'password'] [-P 'Port'] [-i] [-c 'config file']\n\n"
    << "Options:\n"
    << "-h  --help\tthis page\n"
    << "-n  --nocheck\tignore amaroK version during startup\n"
    << "-d  --daemon\tstart in daemon mode\n"
    << "-P  --Port\tdefines server port [default=" << VDRAMGW_PORT << "]\n"
    << "-u  --user\tset the server login username\n"
    << "-p  --passwd\tset the server login password\n"
    << "-i  --info\tshow server info messages\n"
    << "-c  --config\tstart with configuration file\n\n";

}
